from kik.resource import Resource


class Keyboard(Resource):
    """
    Parent class for all keyboards.
    """
    def __init__(self, type, to=None, hidden=None):
        self.type = type
        self.to = to
        self.hidden = hidden

    @classmethod
    def property_mapping(cls):
        return {
            'to': 'to',
            'type': 'type',
            'hidden': 'hidden',
        }


class SuggestedResponseKeyboard(Keyboard):
    """
    A suggested response keyboard as documented at `<https://dev.kik.com/#/docs/messaging#keyboards>`_.

    :param to: (optional) User who will see this keyboard. If None, the keyboard will be shown to all users who don't
        have another keyboard set.
    :type to: str
    :param hidden: (optional) If True, this keyboard will be hidden until the user chooses to see suggested responses.
    :type hidden: bool
    :param responses: (optional) A list of :class:`SuggestedResponse <kik.message.responses.SuggestedResponse>`.
        Defaults to an empty list.
    :type responses: list[kik.message.responses.SuggestedResponse]
    """
    def __init__(self, to=None, hidden=None, responses=None):
        super(SuggestedResponseKeyboard, self).__init__(type='suggested', to=to, hidden=hidden)
        if responses:
            self.responses = responses
        else:
            self.responses = []

    def to_json(self):
        output_json = super(SuggestedResponseKeyboard, self).to_json()
        output_json['responses'] = [response.to_json() for response in self.responses]
        return output_json



from kik.messages.message import Message


class KeyboardMessage(Message):
    """
    Parent class for messages that support keyboards.
    """
    def __init__(self, keyboards=None, **kwargs):
        super(KeyboardMessage, self).__init__(**kwargs)
        if keyboards:
            self.keyboards = keyboards
        else:
            self.keyboards = []

    def to_json(self):
        output_json = super(KeyboardMessage, self).to_json()
        if len(self.keyboards) > 0:
            output_json['keyboards'] = [keyboard.to_json() for keyboard in self.keyboards]

        return output_json



from kik.messages.message import Message


class StartChattingMessage(Message):
    """
    A full start-chatting message object, as documented at `<https://dev.kik.com/#/docs/messaging#start-chatting>`_.
    """
    def __init__(self, **kwargs):
        super(StartChattingMessage, self).__init__(type='start-chatting', **kwargs)



from kik.resource import Resource


class Message(Resource):
    """
    Parent class for all messages.
    """
    def __init__(self, type, to=None, id=None, chat_id=None, mention=None, participants=None, from_user=None,
                 delay=None, read_receipt_requested=None, timestamp=None):
        self.type = type
        self.to = to
        self.id = id
        self.chat_id = chat_id
        self.mention = mention
        self.participants = participants
        self.from_user = from_user
        self.delay = delay
        self.read_receipt_requested = read_receipt_requested
        self.timestamp = timestamp

    @classmethod
    def property_mapping(cls):
        return {
            'type': 'type',
            'id': 'id',
            'chat_id': 'chatId',
            'mention': 'mention',
            'participants': 'participants',
            'to': 'to',
            'from_user': 'from',
            'delay': 'delay',
            'read_receipt_requested': 'readReceiptRequested',
            'timestamp': 'timestamp'
        }



from kik.messages.message import Message


class ScanDataMessage(Message):
    """
    A full scan-data message object, as documented at `<https://dev.kik.com/#/docs/messaging#scan-data>`_.
    """
    def __init__(self, data=None, **kwargs):
        super(ScanDataMessage, self).__init__(type='scan-data', **kwargs)
        self.data = data

    @classmethod
    def property_mapping(cls):
        mapping = super(ScanDataMessage, cls).property_mapping()
        mapping.update({
            'data': 'data'
        })
        return mapping



from kik.messages.message import Message
from kik.messages.attribution import CustomAttribution


class AttributableMessage(Message):
    """
    Parent class for messages that support attribution.
    """
    def __init__(self, attribution=None, **kwargs):
        super(AttributableMessage, self).__init__(**kwargs)
        self.attribution = attribution

    def to_json(self):
        output_json = super(AttributableMessage, self).to_json()
        if self.attribution is not None:
            output_json['attribution'] = self.attribution.to_json()

        return output_json

    @classmethod
    def from_json(cls, json):
        message = super(AttributableMessage, cls).from_json(json)

        if 'attribution' in json:
            message.attribution = CustomAttribution.from_json(json['attribution'])

        return message



from kik.messages.is_typing import IsTypingMessage
from kik.messages.link import LinkMessage
from kik.messages.picture import PictureMessage
from kik.messages.receipts import ReceiptMessage, ReadReceiptMessage, DeliveryReceiptMessage
from kik.messages.scan_data import ScanDataMessage
from kik.messages.start_chatting import StartChattingMessage
from kik.messages.sticker import StickerMessage
from kik.messages.text import TextMessage
from kik.messages.unknown import UnknownMessage
from kik.messages.video import VideoMessage
from kik.messages.utils import messages_from_json
from kik.messages.keyboards import SuggestedResponseKeyboard
from kik.messages.responses import TextResponse
from kik.messages.attribution import CustomAttribution, PresetAttributions
from kik.messages.message import Message


__all__ = ['TextMessage', 'LinkMessage', 'PictureMessage', 'ScanDataMessage', 'StartChattingMessage', 'StickerMessage',
           'VideoMessage', 'IsTypingMessage', 'ReceiptMessage', 'ReadReceiptMessage', 'DeliveryReceiptMessage',
           'UnknownMessage', 'SuggestedResponseKeyboard', 'TextResponse', 'messages_from_json', 'CustomAttribution',
           'PresetAttributions', 'Message']



from kik.messages.keyboard_message import KeyboardMessage
from kik.messages.attributable_message import AttributableMessage


class PictureMessage(KeyboardMessage, AttributableMessage):
    """
    A full picture message object, as documented at `<https://dev.kik.com/#/docs/messaging#picture>`_.
    """
    def __init__(self, to=None, chat_id=None, pic_url=None, keyboards=None, attribution=None, mention=None,
                 delay=None, **kwargs):
        super(PictureMessage, self).__init__(type='picture', to=to, chat_id=chat_id, mention=mention, delay=delay,
                                             keyboards=keyboards, attribution=attribution, **kwargs)
        self.pic_url = pic_url

    @classmethod
    def property_mapping(cls):
        mapping = super(PictureMessage, cls).property_mapping()
        mapping.update({
            'pic_url': 'picUrl'
        })
        return mapping



from kik.resource import Resource


class Configuration(Resource):
    """
    Model for your bot's configuration.

    :param webhook: URL the API will send incoming messages to
    :type webhook: str
    :param features: Feature flags to set
    :type features: dict
    """
    def __init__(self, webhook, features=None):
        self.webhook = webhook
        if features is None:
            features = {}
        self.features = features

    @classmethod
    def property_mapping(cls):
        return {
            'webhook': 'webhook',
            'features': 'features'
        }



from six import iteritems


class Resource(object):
    def __eq__(self, other):
        if type(other) is type(self):
            return self.__dict__ == other.__dict__
        return False

    def __ne__(self, other):
        return not self == other

    def to_json(self):
        output_json = {}
        for obj_key, json_key in iteritems(self.property_mapping()):
            attr = getattr(self, obj_key)
            if attr is not None:
                output_json[json_key] = attr
        return output_json

    @classmethod
    def from_json(cls, json):
        mapping = {v: k for k, v in iteritems(cls.property_mapping())}
        return cls(**{mapping[key]: value for key, value in iteritems(json) if key in mapping})

    @classmethod
    def property_mapping(cls):
        """
        A map of property name to json key name for properties that can be simply serialized to/from json
        (no objects, etc.)
        """
        raise NotImplementedError('Resource objects must define a property_mapping')



from kik.messages.keyboard_message import KeyboardMessage
from kik.messages.attributable_message import AttributableMessage


class LinkMessage(KeyboardMessage, AttributableMessage):
    """
    A full link message object, as documented at `<https://dev.kik.com/#/docs/messaging#link>`_.
    """
    def __init__(self, to=None, chat_id=None, url=None, title=None, text=None, pic_url=None, no_forward=None,
                 kik_js_data=None, keyboards=None, attribution=None, mention=None, delay=None, **kwargs):
        super(LinkMessage, self).__init__(type='link', to=to, chat_id=chat_id, mention=mention, delay=delay,
                                          keyboards=keyboards, attribution=attribution, **kwargs)
        self.url = url
        self.title = title
        self.text = text
        self.pic_url = pic_url
        self.no_forward = no_forward
        self.kik_js_data = kik_js_data

    @classmethod
    def property_mapping(cls):
        mapping = super(LinkMessage, cls).property_mapping()
        mapping.update({
            'url': 'url',
            'title': 'title',
            'text': 'text',
            'no_forward': 'noForward',
            'kik_js_data': 'kikJsData',
            'pic_url': 'picUrl'
        })
        return mapping



from kik import messages
from kik.api import KikApi
from kik.code import Code
from kik.configuration import Configuration
from kik.error import KikError
from kik.user import User

__all__ = ['messages', 'KikApi', 'Code', 'Configuration', 'KikError', 'User']



from kik.messages.keyboard_message import KeyboardMessage


class TextMessage(KeyboardMessage):
    """
    A full link message object, as documented at `<https://dev.kik.com/#/docs/messaging#text>`_.
    """
    def __init__(self, to=None, chat_id=None, body=None, keyboards=None, mention=None, delay=None, type_time=None,
                 **kwargs):
        super(TextMessage, self).__init__(type='text', to=to, chat_id=chat_id, mention=mention, delay=delay,
                                          keyboards=keyboards, **kwargs)
        self.body = body
        self.type_time = type_time

    @classmethod
    def property_mapping(cls):
        mapping = super(TextMessage, cls).property_mapping()
        mapping.update({
            'body': 'body',
            'type_time': 'typeTime'
        })
        return mapping



from kik.messages.message import Message


class IsTypingMessage(Message):
    """
    A full is-typing message object, as documented at `<https://dev.kik.com/#/docs/messaging#is-typing>`_.
    """
    def __init__(self, is_typing=None, **kwargs):
        super(IsTypingMessage, self).__init__(type='is-typing', **kwargs)
        self.is_typing = is_typing

    @classmethod
    def property_mapping(cls):
        mapping = super(IsTypingMessage, cls).property_mapping()
        mapping.update({
            'is_typing': 'isTyping'
        })
        return mapping



from kik.resource import Resource

HOSTED_URL = 'https://api.kik.com/v1/code/{id}{color_query}'


class Code(Resource):
    """
    Model representing a Kik Code. Can be instantiated with a Kik Code ID, or created using
    :func:`kik.KikApi.create_code`.

    :ivar id: The ID for the Kik Code.
    :vartype id: str
    """
    class Colors(object):
        """
        Kik Code color mapping, taken from `<https://dev.kik.com/#/docs/messaging#kik-code-colors>`_
        """
        KIK_BLUE = 0
        TURQUOISE = 1
        MINT = 2
        FOREST = 3
        KIK_GREEN = 4
        SUNSHINE = 5
        ORANGE_CREAMSICLE = 6
        BLOOD_ORANGE = 7
        CANDY_APPLE_RED = 8
        SALMON = 9
        CORAL = 10
        CRANBERRY = 11
        LAVENDER = 12
        ROYAL_PURPLE = 13
        MARINE = 14
        STEEL = 15

    def __init__(self, id, **kwargs):
        super(Code, self).__init__()
        self.id = id

    def url(self, color=None):
        """
        Returns the URL for the Kik Code.

        :param color: (optional) Sets the color of the Kik Code. For options see :class:`kik.Code.Colors`
        :type color: int
        """
        color_query = '?c={}'.format(color) if color else ''
        return HOSTED_URL.format(id=self.id, color_query=color_query)

    @classmethod
    def property_mapping(cls):
        mapping = {
            'id': 'id',
        }
        return mapping



from kik.messages.message import Message


class ReceiptMessage(Message):
    """
    Parent class for all receipts.
    """
    def __init__(self, message_ids=None, **kwargs):
        super(ReceiptMessage, self).__init__(**kwargs)
        self.message_ids = message_ids

    @classmethod
    def property_mapping(cls):
        mapping = super(ReceiptMessage, cls).property_mapping()
        mapping.update({
            'message_ids': 'messageIds'
        })
        return mapping


class ReadReceiptMessage(ReceiptMessage):
    """
    A full read-receipt message object, as documented at `<https://dev.kik.com/#/docs/messaging#receipts>`_.
    """
    def __init__(self, **kwargs):
        super(ReadReceiptMessage, self).__init__(type='read-receipt', **kwargs)


class DeliveryReceiptMessage(ReceiptMessage):
    """
    A full delivery-receipt message object, as documented at `<https://dev.kik.com/#/docs/messaging#receipts>`_.
    """
    def __init__(self, **kwargs):
        super(DeliveryReceiptMessage, self).__init__(type='delivery-receipt', **kwargs)



from kik.resource import Resource


class SuggestedResponse(Resource):
    """
    Base class for all responses for :class:`SuggestedResponseKeyboard<kik.messages.SuggestedResponseKeyboard>`.
    """
    def __init__(self, type):
        self.type = type

    @classmethod
    def property_mapping(cls):
        return {'type': 'type'}


class TextResponse(SuggestedResponse):
    """
    A text response, as documented at `<https://dev.kik.com/#/docs/messaging#suggested-response-keyboard>`_.
    """
    def __init__(self, body):
        super(TextResponse, self).__init__(type='text')
        self.body = body

    @classmethod
    def property_mapping(cls):
        mapping = super(TextResponse, cls).property_mapping()
        mapping.update({'body': 'body'})
        return mapping



import base64
import hashlib
import hmac
import json

import requests

from kik.code import Code
from kik.configuration import Configuration
from kik.error import KikError
from kik.user import User

ROOT_URL = 'https://api.kik.com{}'


class KikApi(object):
    """
    Generic API Client for all Kik API features.

    :param bot: Your bot's username
    :type bot: str
    :param api_key: Your bot's API key
    :type api_key: str
    """
    def __init__(self, bot, api_key):
        super(KikApi, self).__init__()
        self.bot = bot
        self.api_key = api_key

    def send_messages(self, messages):
        """
        Sends a batch of messages.

        :param messages: List of :class:`Message <kik.messages.Message>` to be sent.
        :type messages: list[kik.messages.Message]
        :returns: A dict containing the response from the API
        :rtype: dict

        .. note:: Subject to limits on the number of messages sent, documented at
            `<https://dev.kik.com/#/docs/messaging#sending-messages>`_.

        Usage:

        >>> from kik import KikApi
        >>> from kik.messages import TextMessage
        >>> kik = KikApi(BOT_USERNAME, BOT_API_KEY)
        >>> kik.send_messages([
        >>>     TextMessage(
        >>>         to='ausername',
        >>>         chat_id='2e566cf66b07d9622053b2f0e44dd14926d89a6d61adf496844781876d62cca6',
        >>>         body='Some Text'
        >>>     )
        >>> ])
        {}
        """
        response = requests.post(
            ROOT_URL.format('/v1/message'),
            auth=(self.bot, self.api_key),
            timeout=60,
            headers={
                'Content-Type': 'application/json'
            },
            data=json.dumps({'messages': [m.to_json() for m in messages]})
        )

        if response.status_code != 200:
            raise KikError(response.text)

        return response.json()

    def send_broadcast(self, messages):
        """
        Sends a batch of messages though the broadcast API.

        :param messages: List of :class:`Message <kik.messages.message.Message>` to be sent.
        :type messages: list[kik.messages.Message]
        :returns: A dict containing the response from the API.
        :rtype: dict

        .. note:: Subject to limits on the number of messages sent, documented at
            `<https://dev.kik.com/#/docs/messaging#broadcasting>`_.

        Usage:

        >>> from kik import KikApi
        >>> from kik.messages import TextMessage
        >>> kik = KikApi(BOT_USERNAME, BOT_API_KEY)
        >>> kik.send_broadcast([
        >>>     TextMessage(
        >>>         to='ausername',
        >>>         chat_id='2e566cf66b07d9622053b2f0e44dd14926d89a6d61adf496844781876d62cca6',
        >>>         body='Some Text'
        >>>     )
        >>> ])
        {}
        """
        response = requests.post(
            ROOT_URL.format('/v1/broadcast'),
            auth=(self.bot, self.api_key),
            timeout=60,
            headers={
                'Content-Type': 'application/json'
            },
            data=json.dumps({'messages': [m.to_json() for m in messages]})
        )

        if response.status_code != 200:
            raise KikError(response.text)

        return response.json()

    def get_user(self, username):
        """
        Gets a user's profile data.

        :param username: List of :class:`Message <kik.messages.Message>` to be sent.
        :type username: str
        :returns: A :class:`User <kik.User>` containing the user's profile data.
        :rtype: kik.User

        .. note:: In order to fetch a user's profile, the user must be a subscriber to your bot

        Usage:

        >>> from kik import KikApi
        >>> kik = KikApi(BOT_USERNAME, BOT_API_KEY)
        >>> kik.get_user('aleem')
        <kik.User>
        """
        response = requests.get(
            ROOT_URL.format('/v1/user/{}'.format(username)),
            auth=(self.bot, self.api_key),
            timeout=60
        )

        if response.status_code != 200:
            raise KikError(response.text)

        content = response.json()

        return User.from_json(content)

    def create_code(self, data=None):
        """
        Creates a Kik Code for your bot.

        :param data: (optional) Data to embed in the code, which will be returned in a `scan-data` message when the
            code is scanned.
        :type data: dict, str
        :returns: A :class:`Code <kik.Code>` representing the code.
        :rtype: kik.Code

        Usage:

        >>> from kik import KikApi
        >>> kik = KikApi(BOT_USERNAME, BOT_API_KEY)
        >>> kik.create_code(data='somedata')
        <kik.Code>
        """
        if data is not None:
            payload = json.dumps({'data': json.dumps(data) if isinstance(data, dict) else data})
        else:
            payload = json.dumps({})

        response = requests.post(
            ROOT_URL.format('/v1/code'),
            auth=(self.bot, self.api_key),
            headers={
                'Content-Type': 'application/json'
            },
            data=payload,
            timeout=60
        )

        if response.status_code != 200:
            raise KikError(response.text)

        content = response.json()

        return Code.from_json(content)

    def get_configuration(self):
        """
        Retrieves your bot's configuration

        :returns: A :class:`Configuration <kik.Configuration>` representing the configuration data.
        :rtype: kik.Configuration

        Usage:

        >>> from kik import KikApi
        >>> kik = KikApi(BOT_USERNAME, BOT_API_KEY)
        >>> kik.get_configuration()
        <kik.Configuration>
        """
        response = requests.get(
            ROOT_URL.format('/v1/config'),
            auth=(self.bot, self.api_key),
            timeout=60
        )

        if response.status_code != 200:
            raise KikError(response.text)

        content = response.json()

        return Configuration.from_json(content)

    def set_configuration(self, config):
        """
        Sets your bot's configuration

        :param config: A :class:`Configuration<kik.Configuration>` containing your bot's new configuration
        :type config: kik.Configuration
        :returns: A dict containing the response from the API.
        :rtype: dict

        Usage:

        >>> from kik import KikApi, Configuration
        >>> kik = KikApi(BOT_USERNAME, BOT_API_KEY)
        >>> config = Configuration(webhook='https://example.com/incoming')
        >>> kik.set_configuration(config)
        {}
        """
        response = requests.post(
            ROOT_URL.format('/v1/config'),
            auth=(self.bot, self.api_key),
            headers={
                'Content-Type': 'application/json'
            },
            data=json.dumps(config.to_json()),
            timeout=60
        )

        if response.status_code != 200:
            raise KikError(response.text)

        return response.json()

    def verify_signature(self, signature, body):
        """
        Verifies that a request body correctly matches the header signature.
        For more on signatures see `<https://dev.kik.com/#/docs/messaging#receiving-messages>`_.
        To see this in action, see app.py
        """
        return signature == base64.b16encode(hmac.new(str(self.api_key), body, hashlib.sha1).digest())



from kik.messages import TextMessage, LinkMessage, PictureMessage, VideoMessage, StartChattingMessage, \
    ScanDataMessage, StickerMessage, IsTypingMessage, DeliveryReceiptMessage, ReadReceiptMessage, UnknownMessage

incoming_type_mapping = {
    'text': TextMessage,
    'link': LinkMessage,
    'picture': PictureMessage,
    'video': VideoMessage,
    'start-chatting': StartChattingMessage,
    'scan-data': ScanDataMessage,
    'sticker': StickerMessage,
    'is-typing': IsTypingMessage,
    'delivery-receipt': DeliveryReceiptMessage,
    'read-receipt': ReadReceiptMessage
}


def messages_from_json(messages):
    """
    Converts incoming JSON format messages into message objects.

    :param messages: A list of messages in JSON format.
    :type message: list[dict].
    :returns: A list of messages as Python classes.
    :rtype: list[kik.messages.Message].
    """
    message_objects = []
    for message in messages:
        msg_type = incoming_type_mapping.get(message['type'], UnknownMessage)
        if msg_type is not UnknownMessage:
            # Unknown message types want to keep the type param, as it's not otherwise accessible.
            del message['type']
        message_objects.append(msg_type.from_json(message))
    return message_objects



import codecs
import os

from distutils.core import setup

HERE = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    """
    Build an absolute path from *parts* and and return the contents of the
    resulting file.  Assume UTF-8 encoding.
    """
    with codecs.open(os.path.join(HERE, *parts), "rb", "utf-8") as f:
        return f.read()


setup(
    name='kik',
    version='1.0.6',
    packages=['kik', 'kik.messages'],
    package_dir={
        'kik': 'kik',
        'kik.messages': 'kik/messages'
    },

    author='kik',
    author_email='bots@kik.com',

    url='https://dev.kik.com',

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Topic :: Software Development :: Libraries',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4'
    ],

    long_description=read("README.rst"),

    install_requires=[
        'requests>=2.3.0',
        'six==1.10.0'
    ]
)



from kik.messages.keyboard_message import KeyboardMessage
from kik.messages.attributable_message import AttributableMessage


class VideoMessage(KeyboardMessage, AttributableMessage):
    """
    A full video message object, as documented at `<https://dev.kik.com/#/docs/messaging#video>`_.
    """
    def __init__(self, to=None, chat_id=None, video_url=None, loop=None, muted=None, autoplay=None, no_save=None,
                 keyboards=None, attribution=None, mention=None, delay=None, **kwargs):
        super(VideoMessage, self).__init__(type='video', to=to, chat_id=chat_id, mention=mention, delay=delay,
                                           keyboards=keyboards, attribution=attribution, **kwargs)
        self.video_url = video_url
        self.loop = loop
        self.muted = muted
        self.autoplay = autoplay
        self.no_save = no_save

    @classmethod
    def property_mapping(cls):
        mapping = super(VideoMessage, cls).property_mapping()
        mapping.update({
            'video_url': 'videoUrl',
            'loop': 'loop',
            'muted': 'muted',
            'autoplay': 'autoplay',
            'no_save': 'noSave'
        })
        return mapping



from kik.messages.message import Message


class StickerMessage(Message):
    """
    A full sticker message object, as documented at `<https://dev.kik.com/#/docs/messaging#sticker>`_.
    """
    def __init__(self, sticker_pack_id=None, sticker_url=None, **kwargs):
        super(StickerMessage, self).__init__(type='sticker', **kwargs)
        self.sticker_pack_id = sticker_pack_id
        self.sticker_url = sticker_url

    @classmethod
    def property_mapping(cls):
        mapping = super(StickerMessage, cls).property_mapping()
        mapping.update({
            'sticker_pack_id': 'stickerPackId',
            'sticker_url': 'stickerUrl'
        })
        return mapping



from kik.resource import Resource


class User(Resource):
    """
    Model representing a Kik user's profie information. Created using :func:`kik.KikApi.get_user`.

    :ivar first_name: The user's first name
    :vartype first_name: str
    :ivar last_name: The user's last name
    :vartype last_name: str
    :ivar profile_pic_url: URL for the users's profile picture
    :vartype profile_pic_url: str
    :ivar profile_pic_last_modified: Timestamp indicating when the user's profile picture was
        last modified, to allow for caching
    :vartype: profile_last_modified: int
    """
    def __init__(self, first_name, last_name, profile_pic_url=None, profile_pic_last_modified=None, **kwargs):
        super(User, self).__init__()
        self.first_name = first_name
        self.last_name = last_name
        self.profile_pic_url = profile_pic_url
        self.profile_pic_last_modified = profile_pic_last_modified

    @classmethod
    def property_mapping(cls):
        mapping = {
            'first_name': 'firstName',
            'last_name': 'lastName',
            'profile_pic_url': 'profilePicUrl',
            'profile_pic_last_modified': 'profilePicLastModified'
        }
        return mapping



from kik.messages.message import Message


class UnknownMessage(Message):
    """
    This message type is returned by the message factory when it encounters an unknown message type.

    It's `type` attribute is set to the type of the message, and it's `raw_message` attribute contains the raw JSON
    message received
    """
    @classmethod
    def from_json(cls, json):
        message = super(UnknownMessage, cls).from_json(json)
        message.raw_message = json
        return message

    @classmethod
    def property_mapping(cls):
        mapping = super(UnknownMessage, cls).property_mapping()
        mapping.update({
            'type': 'type'
        })
        return mapping



from kik.resource import Resource


class Attribution(Resource):
    """
    Parent class for all attribution types
    """
    pass


class CustomAttribution(Attribution):
    """
    Attribution class for custom attributions, as documented at `<https://dev.kik.com/#/docs/messaging#attribution>`_

    Usage:

    >>> from kik.messages import CustomAttribution, LinkMessage
    >>> message = LinkMessage()
    >>> message.attribution = CustomAttribution(
    >>>     name='A Name',
    >>>     icon_url='http://foo.bar/anicon'
    >>> )
    """
    def __init__(self, name=None, icon_url=None):
        self.name = name
        self.icon_url = icon_url

    @classmethod
    def property_mapping(cls):
        return {
            'name': 'name',
            'icon_url': 'iconUrl'
        }


class PresetAttribution(Attribution):
    """
    Attribution class for the preset attribution types (e.g. "gallery" or "camera")
    """
    def __init__(self, preset_name):
        self.preset_name = preset_name

    def to_json(self):
        return self.preset_name


class PresetAttributions(object):
    """
    List of preset attribution types.

    Valid only on :class:`PictureMessage <kik.messages.PictureMessage>` and
        :class:`VideoMessage <kik.messages.VideoMessage>`.

    :cvar GALLERY: Makes the message appear to be from a user's gallery.
    :vartype GALLERY: kik.message.attribution.PresetAttribution
    :cvar CAMERA: Makes the message appear to be from a camera.
    :vartype CAMERA: kik.message.attribution.PresetAttribution

    Usage:

    >>> from kik.messages import PresetAttributions, PictureMessage
    >>> message = PictureMessage()
    >>> message.attribution = PresetAttributions.CAMERA
    """
    GALLERY = PresetAttribution('gallery')
    CAMERA = PresetAttribution('camera')
